

set -ex



test -f "${PREFIX}/lib/libscotch${SHLIB_EXT}"
test -f "${PREFIX}/lib/libscotch-6${SHLIB_EXT}"
test -f "${PREFIX}/lib/libscotcherr${SHLIB_EXT}"
test -f "${PREFIX}/lib/libscotcherr-6${SHLIB_EXT}"
test -f "${PREFIX}/lib/libscotcherrexit${SHLIB_EXT}"
test -f "${PREFIX}/lib/libscotcherrexit-6${SHLIB_EXT}"
test -f "${PREFIX}/lib/libscotchmetis${SHLIB_EXT}"
test -f "${PREFIX}/lib/libscotchmetis-6${SHLIB_EXT}"
test -f "${PREFIX}/lib/libesmumps${SHLIB_EXT}"
test -f "${PREFIX}/lib/libesmumps-6${SHLIB_EXT}"
test -f "${PREFIX}/include/scotch.h"
test -f "${PREFIX}/include/scotchf.h"
test -f "${PREFIX}/include/esmumps.h"
test -f "${PREFIX}/include/scotch/metis.h"
test ! -f "${PREFIX}/include/metis.h"
exit 0
